####################################
# names of the ouputed files:

# main output folder
INPUT_MUTS = 'input_mutations.tsv'
TRANSVAR_REVPROT_INPUT = 'transvar_revprotein_input.tsv'  # note that _ens and _refseq will be created
TRANSVAR_REVPROT_OUTPUT = 'transvar_revprotein_output.tsv'
TRANSVAR_DNA_INPUT = 'transvar_dna_input.tsv'
TRANSVAR_DNA_OUTPUT = 'transvar_dna_output.tsv'
ANNOTATED_MUTS = 'annotated_mutations.tsv'
CADD_INPUT = 'cadd_input.tsv.gz'
CADD_OUTPUT = 'cadd_output.tsv.gz'
METADATA_MUTS = 'metadata_mutations.tsv'
DRIVER_MUTS = 'driver_prediction_mutations.tsv'
FINAL_MUTS = 'mutation_analysis.tsv'
REPORT = 'summary.tsv'

PANNO_NOT_MAPPED = 'not_mapped_protein_mutations.tsv'
GANNO_NOT_MAPPED = 'not_mapped_dna_mutations.tsv'
MALFORMED_INPUT = 'malformed_input.tsv'

FS_NOT_MAPPED = 'not_mapped_frameshift.tsv'

####################################
# names of the possible input formats

VCF_FORMAT = 'vcf'
FREE_FORMAT = 'gtf'  #genomic tabulated format
GDNA_FORMAT = 'gdna'
PROT_FORMAT = 'prot'

####################################
# name of columns

# these are not outputed by the pipeline but required as input depending on the format
SAMPLE = 'sample'
CHR = 'chr'
POS = 'pos'
REF = 'ref'
ALT = 'alt'
GDNA = 'gdna'  # this is also used as an output column
PROT = 'protein'  # this is also used as an output column
STRAND = 'strand'

# this can not be changed since is the name used by transvar
GENE = 'gene'  # remember that this is for gene symbol

# columns created by the pipeline
CONSEQUENCE_TYPE = 'consequence'
CDNA = 'cdna'
AA_POS = 'protein_pos'
AA_CHANGE = 'protein_change'
TRANSCRIPT = 'transcript'
EXON = 'exon'
INTERNAL_ID = 'default_id'

# only when input is protein
REV_TRANSCRIPT = 'reverse_transcript'
REV_MAP_ISSUE = 'revmap_issue'

# oncodriveMUT columns
KNOWN_ONCOGENIC = 'known_oncogenic'
KNOWN_PREDISPOSING = 'known_predisposing'
PFAM_DOMAIN = 'Pfam_domain'
TRANSCRIPT_EXONS = 'transcript_exons'
TRANSCRIPT_AMINOACIDS = 'transcript_aminoacids'
EXAC_AF = 'exac_af'

CADD_SCORE = 'cadd_phred'
DRIVER_GENE = 'driver_gene'
GENE_ROLE = 'gene_role'
IS_IN_CLUSTER = 'is_in_cluster'
IS_IN_DELICATE_DOMAIN = 'is_in_delicate_domain'
PROTEIN_AFFECTED = 'mutation_location'

MISSENSE_PREDICTION = 'missense_driver_mut_prediction'
DISRUPTING_PREDICTION = 'disrupting_driver_mut_prediction'
INFRAME_PREDICTION = 'inframe_driver_mut_prediction'
DRIVER_MUT_PREDICTION = 'driver_mut_prediction'
DRIVER_MUT_BOOL = 'driver'
DRIVER_MUT_STATEMENT = 'driver_statement'
KNOWN_MATCH = 'known_match' #whether the tumor of the known mutation match with the tumor sample

# columns required in the in silico prescription
CANCER = 'cancer'
ALT_TYPE = 'alt_type'

# the separator to display a list of cancers as a string
TUMORS_SEP = ';'

####################################
# labels of column values

# the generic tumor type label
PANCANCER = 'CANCER'

# the default sample id inserted if none
DEFAULT_SAMPLE = 'tcgi_sample'

# gene driver
TUMOR_DRIVER = 'tumor_driver'
OTHER_TUMOR_DRIVER = 'other_tumor_driver'

IN_CLUSTER = 'in_cluster'
IN_DELICATE_DOMAIN = 'in_delicate_domain'

# protein portion affected
BEFORE_LAST_EXON = 'before_last_exon'
BEFORE_LAST_PORTION = 'before_last_portion'
LAST_PORTION = 'last_portion'

# gene role
ACT = 'Act'
LOF = 'LoF'
AMBIGUOUS = 'ambiguous'

# driver mut boolean
KNOWN = 'known'
PREDICTED = 'predicted'

# driver mut prediction
TIER_1 = 'TIER 1'
TIER_2 = 'TIER 2'
CADD_N_A = 'CADD N/A'
PASSENGER = 'passenger'
CT_NOT_CONSIDERED = 'not protein-affecting'

POLYM = 'polymorphism'

# tags in transvar info column stating bad reverse mapping (panno utility)
INVALID_TRANSVAR_MAP_TAGS = ['no_valid_transcript_found', 'CSQN=Unclassified', 'CSQN=Frameshift;imprecise']

####################################
# oncodriveMUT parameters

# if a max percentage of the input can not be mapped, stop the execution
MAX_NONMAPPED_RATIO = 0.3

# for ct taxonomy in TransVar, check: https://github.com/zwdzwd/transvar/wiki/Interpret-consequence-labels-%28CSQN%29
MISSENSE_SIMPLE_CT = ['Missense']
MISSENSE_COMPLEX_CT = ['MultiAAMissense']
MISSENSE_CT = MISSENSE_SIMPLE_CT + MISSENSE_COMPLEX_CT
FRAMESHIFT_CT = ['Frameshift']
DISRUPTING_CT = ['Nonsense', 'Frameshift', 'CdsStartLoss', 'CdsStopLoss',
                 'SpliceDonorDeletion', 'SpliceAcceptorDeletion', 'SpliceDonorSNV', 'SpliceAcceptorSNV',
                 #'SpliceDonorSubstitution', 'SpliceAcceptorSubstitution',
                 'SpliceDonorBlockSubstitution', 'SpliceAcceptorBlockSubstitution',
                 'SpliceDonorInsertion', 'SpliceAcceptorInsertion']
INFRAME_CT = ['InFrameDeletion', 'InFrameInsertion']
PAM_CT = MISSENSE_CT + DISRUPTING_CT + INFRAME_CT

# oncodriveMUT customizable thresholds
CADD_DEMANDING_TH = 30
CADD_LOOSE_TH = 25
PROTEIN_TIP = 0.05
POLYM_AF = 0.01

# in case an original column has the same name than a pipeline outputed column, put a suffic in the former
ORIGINAL_COLUMNS_SUFFIX = '_orig'
